@@ -60,11 +60,6 @@ extern volatile int ha_used_fds; // Number of FDs we're currently using
  */
 void fd_delete(int fd);
 
-/* Deletes an FD from the fdsets.
- * The file descriptor is kept open.
- */
-void fd_remove(int fd);
-
 /*
  * Take over a FD belonging to another thread.
  * Returns 0 on success, and -1 on failure.
@@ -266,19 +261,6 @@ static inline void fd_may_both(const int fd)
 	HA_ATOMIC_OR(&fdtab[fd].state, FD_EV_READY_RW);
 }
 
-/* Disable readiness when active. This is useful to interrupt reading when it
- * is suspected that the end of data might have been reached (eg: short read).
- * This can only be done using level-triggered pollers, so if any edge-triggered
- * is ever implemented, a test will have to be added here.
- */
-static inline void fd_done_recv(const int fd)
-{
-	/* removing ready never changes polled status */
-	if ((fdtab[fd].state & (FD_EV_ACTIVE_R|FD_EV_READY_R)) != (FD_EV_ACTIVE_R|FD_EV_READY_R) ||
-	    !HA_ATOMIC_BTR(&fdtab[fd].state, FD_EV_READY_R_BIT))
-		return;
-}
-
 /* Report that FD <fd> cannot send anymore without polling (EAGAIN detected). */
 static inline void fd_cant_send(const int fd)
 {
@@ -306,6 +288,18 @@ static inline void fd_want_recv(int fd)
 	updt_fd_polling(fd);
 }
 
+/* Prepare FD <fd> to try to receive, and only create update if fd_updt exists
+ * (essentially for receivers during early boot).
+ */
+static inline void fd_want_recv_safe(int fd)
+{
+	if ((fdtab[fd].state & FD_EV_ACTIVE_R) ||
+	    HA_ATOMIC_BTS(&fdtab[fd].state, FD_EV_ACTIVE_R_BIT))
+		return;
+	if (fd_updt)
+		updt_fd_polling(fd);
+}
+
 /* Prepare FD <fd> to try to send */
 static inline void fd_want_send(int fd)
 {
@@ -446,6 +440,7 @@ static inline void fd_insert(int fd, void *owner, void (*iocb)(int fd), unsigned
 	fdtab[fd].linger_risk = 0;
 	fdtab[fd].cloned = 0;
 	fdtab[fd].et_possible = 0;
+	fdtab[fd].exported = 0;
 #ifdef DEBUG_FD
 	fdtab[fd].event_count = 0;
 #endif
