@@ -47,18 +47,6 @@
 #define OpenSSL_version_num     SSLeay
 #endif
 
-#if ((OPENSSL_VERSION_NUMBER >= 0x1000200fL) && !defined(OPENSSL_NO_TLSEXT) && !defined(LIBRESSL_VERSION_NUMBER) && !defined(OPENSSL_IS_BORINGSSL))
-#define HAVE_SSL_CTX_ADD_SERVER_CUSTOM_EXT
-#endif
-
-#if ((OPENSSL_VERSION_NUMBER >= 0x10002000L) && !defined(LIBRESSL_VERSION_NUMBER))
-#define HAVE_SSL_CTX_get0_privatekey
-#endif
-
-#if (defined(SN_ct_cert_scts) && !defined(OPENSSL_NO_TLSEXT))
-#define HAVE_SSL_SCTL
-#endif
-
 #if (HA_OPENSSL_VERSION_NUMBER < 0x0090800fL)
 /* Functions present in OpenSSL 0.9.8, older not tested */
 static inline const unsigned char *SSL_SESSION_get_id(const SSL_SESSION *sess, unsigned int *sid_length)
@@ -395,5 +383,15 @@ static inline void SSL_CTX_up_ref(SSL_CTX *ctx)
 #define EVP_MD_CTX_free EVP_MD_CTX_destroy
 #endif
 
+/* OpenSSL 1.0.2 and onwards define SSL_CTX_set1_curves_list which is both a
+ * function and a macro. OpenSSL 1.0.2 to 1.1.0 define SSL_CTRL_SET_CURVES_LIST
+ * as a macro, which disappeared from 1.1.1. BoringSSL only has that one and
+ * not the former macro but it does have the function. Let's keep the test on
+ * the macro matching the function name.
+ */
+#if !defined(SSL_CTX_set1_curves_list) && defined(SSL_CTRL_SET_CURVES_LIST)
+#define SSL_CTX_set1_curves_list SSL_CTX_set1_curves_list
+#endif
+
 #endif /* USE_OPENSSL */
 #endif /* _HAPROXY_OPENSSL_COMPAT_H */
