@@ -21,8 +21,11 @@
 #ifndef _HAPROXY_PROTO_SOCKPAIR_H
 #define _HAPROXY_PROTO_SOCKPAIR_H
 
+extern struct proto_fam proto_fam_sockpair;
+
 int recv_fd_uxst(int sock);
 int send_fd_uxst(int fd, int send_fd);
+int sockpair_bind_receiver(struct receiver *rx, char **errmsg);
 
 #endif /* _HAPROXY_PROTO_SOCKPAIR_H  */
 
