@@ -30,14 +30,16 @@
 
 /* some pointer types referenced below */
 struct listener;
+struct receiver;
 struct connection;
 
 /*
  * Custom network family for str2sa parsing.  Should be ok to do this since
  * sa_family_t is standardized as an unsigned integer
  */
-#define AF_CUST_SOCKPAIR     (AF_MAX + 1)
-#define AF_CUST_MAX          (AF_MAX + 2)
+#define AF_CUST_EXISTING_FD  (AF_MAX + 1)
+#define AF_CUST_SOCKPAIR     (AF_MAX + 2)
+#define AF_CUST_MAX          (AF_MAX + 3)
 
 /*
  * Test in case AF_CUST_MAX overflows the sa_family_t (unsigned int)
@@ -55,35 +57,61 @@ struct connection;
 #define CONNECT_DELACK_ALWAYS                   0x00000004 /* Use a delayed ACK */
 #define CONNECT_CAN_USE_TFO                     0x00000008 /* We can use TFO for this connection */
 
-/* This structure contains all information needed to easily handle a protocol.
- * Its primary goal is to ease listeners maintenance. Specifically, the
- * bind_all() primitive must be used before any fork(), and the enable_all()
- * primitive must be called after the fork() to enable all fds. Last, the
- * unbind_all() primitive closes all listeners.
+/* protocol families define standard functions acting on a given address family
+ * for a socket implementation, such as AF_INET/PF_INET for example.
  */
-struct protocol {
-	char name[PROTO_NAME_LEN];			/* protocol name, zero-terminated */
+struct proto_fam {
+	char name[PROTO_NAME_LEN];                      /* family name, zero-terminated */
 	int sock_domain;				/* socket domain, as passed to socket()   */
-	int sock_type;					/* socket type, as passed to socket()     */
-	int sock_prot;					/* socket protocol, as passed to socket() */
 	sa_family_t sock_family;			/* socket family, for sockaddr */
 	socklen_t sock_addrlen;				/* socket address length, used by bind() */
 	int l3_addrlen;					/* layer3 address length, used by hashes */
-	void (*accept)(int fd);				/* generic accept function */
-	int (*bind)(struct listener *l, char *errmsg, int errlen); /* bind a listener */
-	int (*bind_all)(struct protocol *proto, char *errmsg, int errlen); /* bind all unbound listeners */
-	int (*unbind_all)(struct protocol *proto);	/* unbind all bound listeners */
-	int (*enable_all)(struct protocol *proto);	/* enable all bound listeners */
-	int (*disable_all)(struct protocol *proto);	/* disable all bound listeners */
-	int (*connect)(struct connection *, int flags); /* connect function if any, see below for flags values */
+	int (*addrcmp)(const struct sockaddr_storage *, const struct sockaddr_storage *); /* compare addresses (like memcmp) */
+	int (*bind)(struct receiver *rx, char **errmsg); /* bind a receiver */
 	int (*get_src)(int fd, struct sockaddr *, socklen_t, int dir); /* syscall used to retrieve src addr */
 	int (*get_dst)(int fd, struct sockaddr *, socklen_t, int dir); /* syscall used to retrieve dst addr */
-	int (*drain)(int fd);                           /* indicates whether we can safely close the fd */
-	int (*pause)(struct listener *l);               /* temporarily pause this listener for a soft restart */
-	void (*add)(struct listener *l, int port);      /* add a listener for this protocol and port */
+};
 
-	struct list listeners;				/* list of listeners using this protocol (under proto_lock) */
-	int nb_listeners;				/* number of listeners (under proto_lock) */
+/* This structure contains all information needed to easily handle a protocol.
+ * Its primary goal is to ease listeners maintenance. Specifically, the
+ * bind() primitive must be used before any fork(). rx_suspend()/rx_resume()
+ * return >0 on success, 0 if rx stopped, -1 on failure to proceed. rx_* may
+ * be null if the protocol doesn't provide direct access to the receiver.
+ */
+struct protocol {
+	char name[PROTO_NAME_LEN];			/* protocol name, zero-terminated */
+	struct proto_fam *fam;                          /* protocol family */
+	int ctrl_type;                                  /* control layer type (SOCK_STREAM/SOCK_DGRAM) */
+	int sock_domain;				/* socket domain, as passed to socket()   */
+	int sock_type;					/* socket type, as passed to socket()     */
+	int sock_prot;					/* socket protocol, as passed to socket() */
+
+	/* functions acting on the listener */
+	void (*add)(struct listener *l, int port);      /* add a listener for this protocol and port */
+	int (*listen)(struct listener *l, char *errmsg, int errlen); /* start a listener */
+	void (*enable)(struct listener *l);             /* enable receipt of new connections */
+	void (*disable)(struct listener *l);            /* disable receipt of new connections */
+	void (*unbind)(struct listener *l);             /* unbind the listener and possibly its receiver */
+	int (*suspend)(struct listener *l);             /* try to suspend the listener */
+	int (*resume)(struct listener *l);              /* try to resume a suspended listener */
+	struct connection *(*accept_conn)(struct listener *l, int *status); /* accept a new connection */
+
+	/* functions acting on the receiver */
+	void (*rx_enable)(struct receiver *rx);         /* enable receiving on the receiver */
+	void (*rx_disable)(struct receiver *rx);        /* disable receiving on the receiver */
+	void (*rx_unbind)(struct receiver *rx);         /* unbind the receiver, most often closing the FD */
+	int (*rx_suspend)(struct receiver *rx);         /* temporarily suspend this receiver for a soft restart */
+	int (*rx_resume)(struct receiver *rx);          /* try to resume a temporarily suspended receiver */
+	int (*rx_listening)(const struct receiver *rx); /* is the receiver listening ? 0=no, >0=OK, <0=unrecoverable */
+
+	/* default I/O handler */
+	void (*default_iocb)(int fd);                   /* generic I/O handler (typically accept callback) */
+
+	/* functions acting on connections */
+	int (*connect)(struct connection *, int flags); /* connect function if any, see below for flags values */
+
+	struct list receivers;				/* list of receivers using this protocol (under proto_lock) */
+	int nb_receivers;				/* number of receivers (under proto_lock) */
 	struct list list;				/* list of registered protocols (under proto_lock) */
 };
 
