@@ -78,9 +78,20 @@ static inline void session_unown_conn(struct session *sess, struct connection *c
 {
 	struct sess_srv_list *srv_list = NULL;
 
+	/* WT: this currently is a workaround for an inconsistency between
+	 * the link status of the connection in the session list and the
+	 * connection's owner. This should be removed as soon as all this
+	 * is addressed. Right now it's possible to enter here with a non-null
+	 * conn->owner that points to a dead session, but in this case the
+	 * element is not linked.
+	 */
+	if (!LIST_ADDED(&conn->session_list))
+		return;
+
 	if (conn->flags & CO_FL_SESS_IDLE)
 		sess->idle_conns--;
 	LIST_DEL_INIT(&conn->session_list);
+	conn->owner = NULL;
 	list_for_each_entry(srv_list, &sess->srv_list, srv_list) {
 		if (srv_list->target == conn->target) {
 			if (LIST_ISEMPTY(&srv_list->conn_list)) {
@@ -103,7 +114,7 @@ static inline int session_add_conn(struct session *sess, struct connection *conn
 	int found = 0;
 
 	/* Already attach to the session or not the connection owner */
-	if (!LIST_ISEMPTY(&conn->session_list) || conn->owner != sess)
+	if (!LIST_ISEMPTY(&conn->session_list) || (conn->owner && conn->owner != sess))
 		return 1;
 
 	list_for_each_entry(srv_list, &sess->srv_list, srv_list) {
