@@ -548,6 +548,13 @@ static struct server *get_server_rnd(struct stream *s, const struct server *avoi
 			curr = prev;
 	} while (--draws > 0);
 
+	/* if the selected server is full, pretend we have none so that we reach
+	 * the backend's queue instead.
+	 */
+	if (curr &&
+	    (curr->nbpend || (curr->maxconn && curr->served >= srv_dynamic_maxconn(curr))))
+		curr = NULL;
+
 	return curr;
 }
 
@@ -633,14 +640,25 @@ int assign_server(struct stream *s)
 			}
 		}
 	}
-	if (s->be->lbprm.algo & BE_LB_KIND) {
 
+	if (s->be->lbprm.algo & BE_LB_KIND) {
 		/* we must check if we have at least one server available */
 		if (!s->be->lbprm.tot_weight) {
 			err = SRV_STATUS_NOSRV;
 			goto out;
 		}
 
+		/* if there's some queue on the backend, with certain algos we
+		 * know it's because all servers are full.
+		 */
+		if (s->be->nbpend &&
+		    (((s->be->lbprm.algo & (BE_LB_KIND|BE_LB_NEED|BE_LB_PARM)) == BE_LB_ALGO_FAS)||   // first
+		     ((s->be->lbprm.algo & (BE_LB_KIND|BE_LB_NEED|BE_LB_PARM)) == BE_LB_ALGO_RR) ||   // roundrobin
+		     ((s->be->lbprm.algo & (BE_LB_KIND|BE_LB_NEED|BE_LB_PARM)) == BE_LB_ALGO_SRR))) { // static-rr
+			err = SRV_STATUS_FULL;
+			goto out;
+		}
+
 		/* First check whether we need to fetch some data or simply call
 		 * the LB lookup function. Only the hashing functions will need
 		 * some input data in fact, and will support multiple algorithms.
@@ -801,7 +819,7 @@ out_ok:
 			sess_change_server(s, srv);
 		} else {
 			if (may_dequeue_tasks(conn_slot, s->be))
-				process_srv_queue(conn_slot, 0);
+				process_srv_queue(conn_slot);
 		}
 	}
 
@@ -827,7 +845,7 @@ int assign_server_address(struct stream *s)
 
 	DPRINTF(stderr,"assign_server_address : s=%p\n",s);
 
-	if (!sockaddr_alloc(&s->target_addr))
+	if (!sockaddr_alloc(&s->target_addr, NULL, 0))
 		return SRV_STATUS_INTERNAL;
 
 	if ((s->flags & SF_DIRECT) || (s->be->lbprm.algo & BE_LB_KIND)) {
@@ -847,10 +865,8 @@ int assign_server_address(struct stream *s)
 			if (!conn_get_dst(cli_conn)) {
 				/* do nothing if we can't retrieve the address */
 			} else if (cli_conn->dst->ss_family == AF_INET) {
-				((struct sockaddr_in *)s->target_addr)->sin_family = AF_INET;
 				((struct sockaddr_in *)s->target_addr)->sin_addr = ((struct sockaddr_in *)cli_conn->dst)->sin_addr;
 			} else if (cli_conn->dst->ss_family == AF_INET6) {
-				((struct sockaddr_in6 *)s->target_addr)->sin6_family = AF_INET6;
 				((struct sockaddr_in6 *)s->target_addr)->sin6_addr = ((struct sockaddr_in6 *)cli_conn->dst)->sin6_addr;
 			}
 		}
@@ -1040,7 +1056,7 @@ static void assign_tproxy_address(struct stream *s)
 	else
 		return;
 
-	if (!sockaddr_alloc(&srv_conn->src))
+	if (!sockaddr_alloc(&srv_conn->src, NULL, 0))
 		return;
 
 	switch (src->opts & CO_SRC_TPROXY_MASK) {
@@ -1084,7 +1100,7 @@ static void assign_tproxy_address(struct stream *s)
  * (safe or idle connections). The <is_safe> argument means what type of
  * connection the caller wants.
  */
-static struct connection *conn_backend_get(struct server *srv, int is_safe)
+static struct connection *conn_backend_get(struct stream *s, struct server *srv, int is_safe)
 {
 	struct mt_list *mt_list = is_safe ? srv->safe_conns : srv->idle_conns;
 	struct connection *conn;
@@ -1151,6 +1167,7 @@ static struct connection *conn_backend_get(struct server *srv, int is_safe)
 				MT_LIST_DEL_SAFE(elt1);
 				_HA_ATOMIC_ADD(&activity[tid].fd_takeover, 1);
 				found = 1;
+
 				break;
 			}
 		}
@@ -1163,6 +1180,7 @@ static struct connection *conn_backend_get(struct server *srv, int is_safe)
 					found = 1;
 					is_safe = 1;
 					mt_list = srv->safe_conns;
+
 					break;
 				}
 			}
@@ -1174,13 +1192,26 @@ static struct connection *conn_backend_get(struct server *srv, int is_safe)
 		conn = NULL;
  done:
 	if (conn) {
-		conn->idle_time = 0;
 		_HA_ATOMIC_STORE(&srv->next_takeover, (i + 1 == global.nbthread) ? 0 : i + 1);
+
+		srv_use_conn(srv, conn);
+
 		_HA_ATOMIC_SUB(&srv->curr_idle_conns, 1);
+		_HA_ATOMIC_SUB(conn->flags & CO_FL_SAFE_LIST ? &srv->curr_safe_nb : &srv->curr_idle_nb, 1);
 		_HA_ATOMIC_SUB(&srv->curr_idle_thr[i], 1);
-		_HA_ATOMIC_SUB(is_safe ? &srv->curr_safe_nb : &srv->curr_idle_nb, 1);
+		conn->flags &= ~CO_FL_LIST_MASK;
 		__ha_barrier_atomic_store();
-		LIST_ADDQ(&srv->available_conns[tid], mt_list_to_list(&conn->list));
+
+		if ((s->be->options & PR_O_REUSE_MASK) == PR_O_REUSE_SAFE &&
+		    conn->mux->flags & MX_FL_HOL_RISK) {
+			/* attach the connection to the session private list
+			 */
+			conn->owner = s->sess;
+			session_add_conn(s->sess, conn, conn->target);
+		}
+		else {
+			LIST_ADDQ(&srv->available_conns[tid], mt_list_to_list(&conn->list));
+		}
 	}
 	return conn;
 }
@@ -1205,13 +1236,10 @@ int connect_server(struct stream *s)
 	struct connection *cli_conn = objt_conn(strm_orig(s));
 	struct connection *srv_conn = NULL;
 	struct conn_stream *srv_cs = NULL;
-	struct sess_srv_list *srv_list;
 	struct server *srv;
 	int reuse = 0;
-	int reuse_orphan = 0;
 	int init_mux = 0;
 	int err;
-	int was_unused = 0;
 
 
 	/* This will catch some corner cases such as lying connections resulting from
@@ -1220,21 +1248,9 @@ int connect_server(struct stream *s)
 	si_release_endpoint(&s->si[1]);
 
 	/* first, search for a matching connection in the session's idle conns */
-	list_for_each_entry(srv_list, &s->sess->srv_list, srv_list) {
-		if (srv_list->target == s->target) {
-			list_for_each_entry(srv_conn, &srv_list->conn_list, session_list) {
-				if (conn_xprt_ready(srv_conn) &&
-				    srv_conn->mux && (srv_conn->mux->avail_streams(srv_conn) > 0)) {
-					reuse = 1;
-					break;
-				}
-			}
-			break;
-		}
-	}
-
-	if (!reuse)
-		srv_conn = NULL;
+	srv_conn = session_get_conn(s->sess, s->target);
+	if (srv_conn)
+		reuse = 1;
 
 	srv = objt_server(s->target);
 
@@ -1272,21 +1288,18 @@ int connect_server(struct stream *s)
 				/* we're on the second column of the tables above, let's
 				 * try idle then safe.
 				 */
-				srv_conn = conn_backend_get(srv, 0);
-				was_unused = 1;
+				srv_conn = conn_backend_get(s, srv, 0);
 			}
 			else if (srv->safe_conns &&
 			         ((s->txn && (s->txn->flags & TX_NOT_FIRST)) ||
 				  (s->be->options & PR_O_REUSE_MASK) >= PR_O_REUSE_AGGR) &&
 				 srv->curr_safe_nb > 0) {
-				srv_conn = conn_backend_get(srv, 1);
-				was_unused = 1;
+				srv_conn = conn_backend_get(s, srv, 1);
 			}
 			else if (srv->idle_conns &&
 			         ((s->be->options & PR_O_REUSE_MASK) == PR_O_REUSE_ALWS) &&
 				 srv->curr_idle_nb > 0) {
-				srv_conn = conn_backend_get(srv, 0);
-				was_unused = 1;
+				srv_conn = conn_backend_get(s, srv, 0);
 			}
 			/* If we've picked a connection from the pool, we now have to
 			 * detach it. We may have to get rid of the previous idle
@@ -1294,11 +1307,8 @@ int connect_server(struct stream *s)
 			 * other owner's. That way it may remain alive for others to
 			 * pick.
 			 */
-			if (srv_conn) {
-				reuse_orphan = 1;
+			if (srv_conn)
 				reuse = 1;
-				srv_conn->flags &= ~CO_FL_LIST_MASK;
-			}
 		}
 	}
 
@@ -1344,40 +1354,26 @@ int connect_server(struct stream *s)
 				// see it possibly larger.
 				ALREADY_CHECKED(i);
 
-				HA_SPIN_LOCK(OTHER_LOCK, &idle_conns[i].takeover_lock);
+				HA_SPIN_LOCK(OTHER_LOCK, &idle_conns[tid].takeover_lock);
 				tokill_conn = MT_LIST_POP(&srv->idle_conns[i],
 				    struct connection *, list);
 				if (!tokill_conn)
 					tokill_conn = MT_LIST_POP(&srv->safe_conns[i],
 					    struct connection *, list);
-
 				if (tokill_conn) {
 					/* We got one, put it into the concerned thread's to kill list, and wake it's kill task */
 
 					MT_LIST_ADDQ(&idle_conns[i].toremove_conns,
 					    (struct mt_list *)&tokill_conn->list);
 					task_wakeup(idle_conns[i].cleanup_task, TASK_WOKEN_OTHER);
-					HA_SPIN_UNLOCK(OTHER_LOCK, &idle_conns[i].takeover_lock);
+					HA_SPIN_UNLOCK(OTHER_LOCK, &idle_conns[tid].takeover_lock);
 					break;
 				}
-				HA_SPIN_UNLOCK(OTHER_LOCK, &idle_conns[i].takeover_lock);
+				HA_SPIN_UNLOCK(OTHER_LOCK, &idle_conns[tid].takeover_lock);
 			}
 		}
 
 	}
-	/* If we're really reusing the connection, remove it from the orphan
-	 * list and add it back to the idle list.
-	 */
-	if (reuse) {
-		if (!reuse_orphan) {
-			if (srv_conn->flags & CO_FL_SESS_IDLE) {
-				struct session *sess = srv_conn->owner;
-
-				srv_conn->flags &= ~CO_FL_SESS_IDLE;
-				sess->idle_conns--;
-			}
-		}
-	}
 
 	if (reuse) {
 		if (srv_conn->mux) {
@@ -1405,25 +1401,17 @@ int connect_server(struct stream *s)
 
 	/* no reuse or failed to reuse the connection above, pick a new one */
 	if (!srv_conn) {
-		srv_conn = conn_new();
-		was_unused = 1;
-		if (srv_conn)
-			srv_conn->target = s->target;
+		srv_conn = conn_new(s->target);
 		srv_cs = NULL;
+
+		if (srv_conn) {
+			srv_conn->owner = s->sess;
+			if ((s->be->options & PR_O_REUSE_MASK) == PR_O_REUSE_NEVR)
+				conn_set_private(srv_conn);
+		}
 	}
 
-	if (srv_conn && srv && was_unused) {
-		_HA_ATOMIC_ADD(&srv->curr_used_conns, 1);
-		/* It's ok not to do that atomically, we don't need an
-		 * exact max.
-		 */
-		if (srv->max_used_conns < srv->curr_used_conns)
-			srv->max_used_conns = srv->curr_used_conns;
-
-		if (srv->est_need_conns < srv->curr_used_conns)
-			srv->est_need_conns = srv->curr_used_conns;
-	}
-	if (!srv_conn || !sockaddr_alloc(&srv_conn->dst)) {
+	if (!srv_conn || !sockaddr_alloc(&srv_conn->dst, 0, 0)) {
 		if (srv_conn)
 			conn_free(srv_conn);
 		return SF_ERR_RESOURCE;
@@ -1472,15 +1460,12 @@ int connect_server(struct stream *s)
 		     srv->mux_proto || s->be->mode != PR_MODE_HTTP))
 #endif
 			init_mux = 1;
-#if defined(USE_OPENSSL) && defined(TLSEXT_TYPE_application_layer_protocol_negotiation)
-		else
-			srv_conn->owner = s->sess;
-#endif
+
 		/* process the case where the server requires the PROXY protocol to be sent */
 		srv_conn->send_proxy_ofs = 0;
 
 		if (srv && srv->pp_opts) {
-			srv_conn->flags |= CO_FL_PRIVATE;
+			conn_set_private(srv_conn);
 			srv_conn->flags |= CO_FL_SEND_PROXY;
 			srv_conn->send_proxy_ofs = 1; /* must compute size */
 			if (cli_conn)
@@ -1499,9 +1484,11 @@ int connect_server(struct stream *s)
 			srv_conn->mux->reset(srv_conn);
 	}
 	else {
-		s->flags |= SF_SRV_REUSED;
-		if (!(srv_conn->mux->ctl(srv_conn, MUX_STATUS, NULL) & MUX_STATUS_READY))
-			s->flags |= SF_SRV_REUSED_ANTICIPATED;
+		/* Only consider we're doing reuse if the connection was
+		 * ready.
+		 */
+		if (srv_conn->mux->ctl(srv_conn, MUX_STATUS, NULL) & MUX_STATUS_READY)
+			s->flags |= SF_SRV_REUSED;
 	}
 
 	/* flag for logging source ip/port */
@@ -1526,6 +1513,22 @@ int connect_server(struct stream *s)
 	if (err != SF_ERR_NONE)
 		return err;
 
+#ifdef USE_OPENSSL
+	if (srv && srv->ssl_ctx.sni) {
+		struct sample *smp;
+
+		smp = sample_fetch_as_type(s->be, s->sess, s, SMP_OPT_DIR_REQ | SMP_OPT_FINAL,
+					   srv->ssl_ctx.sni, SMP_T_STR);
+		if (smp_make_safe(smp)) {
+			ssl_sock_set_servername(srv_conn, smp->data.u.str.area);
+			if (!(srv->ssl_ctx.sni->fetch->use & SMP_USE_INTRN) ||
+			    smp->flags & SMP_F_VOLATILE) {
+				conn_set_private(srv_conn);
+			}
+		}
+	}
+#endif /* USE_OPENSSL */
+
 	/* The CO_FL_SEND_PROXY flag may have been set by the connect method,
 	 * if so, add our handshake pseudo-XPRT now.
 	 */
@@ -1548,11 +1551,20 @@ int connect_server(struct stream *s)
 		}
 		/* If we're doing http-reuse always, and the connection is not
 		 * private with available streams (an http2 connection), add it
-		 * to the available list, so that others can use it right away.
+		 * to the available list, so that others can use it right
+		 * away. If the connection is private or we're doing http-reuse
+		 * safe and the mux protocol supports multiplexing, add it in
+		 * the session server list.
 		 */
 		if (srv && ((s->be->options & PR_O_REUSE_MASK) == PR_O_REUSE_ALWS) &&
 		    !(srv_conn->flags & CO_FL_PRIVATE) && srv_conn->mux->avail_streams(srv_conn) > 0)
 			LIST_ADDQ(&srv->available_conns[tid], mt_list_to_list(&srv_conn->list));
+		else if (srv_conn->flags & CO_FL_PRIVATE ||
+		         ((s->be->options & PR_O_REUSE_MASK) == PR_O_REUSE_SAFE &&
+		          srv_conn->mux->flags & MX_FL_HOL_RISK)) {
+			/* If it fail now, the same will be done in mux->detach() callback */
+			session_add_conn(s->sess, srv_conn, srv_conn->target);
+		}
 	}
 
 #if USE_OPENSSL && (defined(OPENSSL_IS_BORINGSSL) || (HA_OPENSSL_VERSION_NUMBER >= 0x10101000L))
@@ -1586,21 +1598,6 @@ int connect_server(struct stream *s)
 			s->be->lbprm.server_take_conn(srv);
 			HA_SPIN_UNLOCK(SERVER_LOCK, &srv->lock);
 		}
-
-#ifdef USE_OPENSSL
-		if (!(s->flags & SF_SRV_REUSED) && srv->ssl_ctx.sni) {
-			struct sample *smp;
-
-			smp = sample_fetch_as_type(s->be, s->sess, s, SMP_OPT_DIR_REQ | SMP_OPT_FINAL,
-						   srv->ssl_ctx.sni, SMP_T_STR);
-			if (smp_make_safe(smp)) {
-				ssl_sock_set_servername(srv_conn,
-							smp->data.u.str.area);
-				srv_conn->flags |= CO_FL_PRIVATE;
-			}
-		}
-#endif /* USE_OPENSSL */
-
 	}
 
 	/* Now handle synchronously connected sockets. We know the stream-int
@@ -1727,7 +1724,7 @@ int srv_redispatch_connect(struct stream *s)
 
 		/* release other streams waiting for this server */
 		if (may_dequeue_tasks(srv, s->be))
-			process_srv_queue(srv, 0);
+			process_srv_queue(srv);
 		return 1;
 	}
 	/* if we get here, it's because we got SRV_STATUS_OK, which also
@@ -1803,7 +1800,7 @@ void back_try_conn_req(struct stream *s)
 			/* release other streams waiting for this server */
 			sess_change_server(s, NULL);
 			if (may_dequeue_tasks(srv, s->be))
-				process_srv_queue(srv, 0);
+				process_srv_queue(srv);
 
 			/* Failed and not retryable. */
 			si_shutr(si);
@@ -2089,7 +2086,7 @@ void back_handle_st_cer(struct stream *s)
 
 	/* we probably have to release last stream from the server */
 	if (objt_server(s->target)) {
-		health_adjust(__objt_server(s->target), HANA_STATUS_L4_ERR);
+		health_adjust(objt_server(s->target), HANA_STATUS_L4_ERR);
 
 		if (s->flags & SF_CURR_SESS) {
 			s->flags &= ~SF_CURR_SESS;
@@ -2131,7 +2128,7 @@ void back_handle_st_cer(struct stream *s)
 		_HA_ATOMIC_ADD(&s->be->be_counters.failed_conns, 1);
 		sess_change_server(s, NULL);
 		if (may_dequeue_tasks(objt_server(s->target), s->be))
-			process_srv_queue(objt_server(s->target), 0);
+			process_srv_queue(objt_server(s->target));
 
 		/* shutw is enough so stop a connecting socket */
 		si_shutw(si);
@@ -2146,18 +2143,6 @@ void back_handle_st_cer(struct stream *s)
 		goto end;
 	}
 
-	/* At this stage, we will trigger a connection retry (with or without
-	 * redispatch). Thus we must release the SI endpoint on the server side
-	 * an close the attached connection. It is especially important to do it
-	 * now if the retry is not immediately performed, to be sure to release
-	 * ressources as soon as possible and to not catch errors from the lower
-	 * layers in an unexpected state (i.e < ST_CONN).
-	 *
-	 * Note: the stream-interface will be switched to ST_REQ, ST_ASS or
-	 * ST_TAR and SI_FL_ERR and SI_FL_EXP flags will be unset.
-	 */
-	si_release_endpoint(&s->si[1]);
-
 	stream_choose_redispatch(s);
 
 	if (si->flags & SI_FL_ERR) {
@@ -2170,8 +2155,6 @@ void back_handle_st_cer(struct stream *s)
 		 */
 
 		int delay = 1000;
-		const int reused = (s->flags & SF_SRV_REUSED) &&
-		                   !(s->flags & SF_SRV_REUSED_ANTICIPATED);
 
 		if (s->be->timeout.connect && s->be->timeout.connect < delay)
 			delay = s->be->timeout.connect;
@@ -2182,7 +2165,7 @@ void back_handle_st_cer(struct stream *s)
 		/* only wait when we're retrying on the same server */
 		if ((si->state == SI_ST_ASS ||
 		     (s->be->lbprm.algo & BE_LB_KIND) != BE_LB_KIND_RR ||
-		     (s->be->srv_act <= 1)) && !reused) {
+		     (s->be->srv_act <= 1)) && !(s->flags & SF_SRV_REUSED)) {
 			si->state = SI_ST_TAR;
 			si->exp = tick_add(now_ms, MS_TO_TICKS(delay));
 		}
@@ -2894,6 +2877,48 @@ smp_fetch_srv_sess_rate(const struct arg *args, struct sample *smp, const char *
 	return 1;
 }
 
+/* set temp integer to the server weight.
+ * Accepts exactly 1 argument. Argument is a server, other types will lead to
+ * undefined behaviour.
+ */
+static int
+smp_fetch_srv_weight(const struct arg *args, struct sample *smp, const char *kw, void *private)
+{
+	struct server *srv = args->data.srv;
+	struct proxy *px = srv->proxy;
+
+	smp->flags = SMP_F_VOL_TEST;
+	smp->data.type = SMP_T_SINT;
+	smp->data.u.sint = (srv->cur_eweight * px->lbprm.wmult + px->lbprm.wdiv - 1) / px->lbprm.wdiv;
+	return 1;
+}
+
+/* set temp integer to the server initial weight.
+ * Accepts exactly 1 argument. Argument is a server, other types will lead to
+ * undefined behaviour.
+ */
+static int
+smp_fetch_srv_iweight(const struct arg *args, struct sample *smp, const char *kw, void *private)
+{
+	smp->flags = SMP_F_VOL_TEST;
+	smp->data.type = SMP_T_SINT;
+	smp->data.u.sint = args->data.srv->iweight;
+	return 1;
+}
+
+/* set temp integer to the server user-specified weight.
+ * Accepts exactly 1 argument. Argument is a server, other types will lead to
+ * undefined behaviour.
+ */
+static int
+smp_fetch_srv_uweight(const struct arg *args, struct sample *smp, const char *kw, void *private)
+{
+	smp->flags = SMP_F_VOL_TEST;
+	smp->data.type = SMP_T_SINT;
+	smp->data.u.sint = args->data.srv->uweight;
+	return 1;
+}
+
 static int sample_conv_nbsrv(const struct arg *args, struct sample *smp, void *private)
 {
 
@@ -2965,6 +2990,9 @@ static struct sample_fetch_kw_list smp_kws = {ILH, {
 	{ "srv_name",      smp_fetch_srv_name,       0,           NULL, SMP_T_STR,  SMP_USE_SERVR, },
 	{ "srv_queue",     smp_fetch_srv_queue,      ARG1(1,SRV), NULL, SMP_T_SINT, SMP_USE_INTRN, },
 	{ "srv_sess_rate", smp_fetch_srv_sess_rate,  ARG1(1,SRV), NULL, SMP_T_SINT, SMP_USE_INTRN, },
+	{ "srv_weight",    smp_fetch_srv_weight,     ARG1(1,SRV), NULL, SMP_T_SINT, SMP_USE_INTRN, },
+	{ "srv_iweight",   smp_fetch_srv_iweight,    ARG1(1,SRV), NULL, SMP_T_SINT, SMP_USE_INTRN, },
+	{ "srv_uweight",   smp_fetch_srv_uweight,    ARG1(1,SRV), NULL, SMP_T_SINT, SMP_USE_INTRN, },
 	{ /* END */ },
 }};
 
