@@ -58,14 +58,6 @@ int cfg_parse_global(const char *file, int linenum, char **args, int kwm)
 		}
 		global.mode |= MODE_MWORKER;
 	}
-	else if (!strcmp(args[0], "debug")) {
-		if (alertif_too_many_args(0, file, linenum, args, &err_code))
-			goto out;
-		global.mode |= MODE_DEBUG;
-		ha_warning("parsing [%s:%d] : '%s' must never be used and will be removed in 2.3. If you need debug mode, please use '-d' on the command line.\n", file, linenum, args[0]);
-		err_code |= ERR_WARN;
-		goto out;
-	}
 	else if (!strcmp(args[0], "noepoll")) {
 		if (alertif_too_many_args(0, file, linenum, args, &err_code))
 			goto out;
@@ -198,7 +190,14 @@ int cfg_parse_global(const char *file, int linenum, char **args, int kwm)
 		global.tune.maxaccept = max;
 	}
 	else if (!strcmp(args[0], "tune.chksize")) {
-		/* Deprecated now */
+		if (alertif_too_many_args(1, file, linenum, args, &err_code))
+			goto out;
+		if (*(args[1]) == 0) {
+			ha_alert("parsing [%s:%d] : '%s' expects an integer argument.\n", file, linenum, args[0]);
+			err_code |= ERR_ALERT | ERR_FATAL;
+			goto out;
+		}
+		global.tune.chksize = atol(args[1]);
 	}
 	else if (!strcmp(args[0], "tune.recv_enough")) {
 		if (alertif_too_many_args(1, file, linenum, args, &err_code))
@@ -1086,34 +1085,22 @@ int cfg_parse_global(const char *file, int linenum, char **args, int kwm)
 				}
 			}
 		} else {
-			/* Mapping at the thread level.
-			 * Either proc and/or thread must be 1 and only 1. All
-			 * other combinations are silently ignored.
+			/* Mapping at the thread level. All threads are retained
+			 * for process 1, and only thread 1 is retained for other
+			 * processes.
 			 */
 			if (thread == 0x1) {
-				int val;
-
 				/* first thread, iterate on processes. E.g. cpu-map 1-4/1 0-3 */
 				for (i = n = 0; i < MAX_PROCS; i++) {
 					/* No mapping for this process */
 					if (!(proc & (1UL << i)))
 						continue;
-
-					if (!autoinc) {
-						val = cpus;
-					}
+					if (!autoinc)
+						global.cpu_map.proc_t1[i] = cpus;
 					else {
 						n += my_ffsl(cpus >> n);
-						val = 1UL << (n - 1);
+						global.cpu_map.proc_t1[i] = (1UL << (n-1));
 					}
-
-					/* For first process, thread[0] is used.
-					 * Use proc_t1[N] for all others
-					 */
-					if (!i)
-						global.cpu_map.thread[0] = val;
-					else
-						global.cpu_map.proc_t1[i] = val;
 				}
 			}
 
@@ -1217,8 +1204,6 @@ int cfg_parse_global(const char *file, int linenum, char **args, int kwm)
 			goto out;
 		if (kwm == KWM_NO)
 			global.tune.options &= ~GTUNE_STRICT_LIMITS;
-		else
-			global.tune.options |= GTUNE_STRICT_LIMITS;
 	}
 	else if (!strcmp(args[0], "localpeer")) {
 		if (alertif_too_many_args(1, file, linenum, args, &err_code))
