@@ -794,17 +794,13 @@ static int wake_srv_chk(struct conn_stream *cs)
 		task_wakeup(check->task, TASK_WOKEN_IO);
 	}
 
-	if (check->result != CHK_RES_UNKNOWN || ret == -1) {
+	if (check->result != CHK_RES_UNKNOWN) {
 		/* Check complete or aborted. If connection not yet closed do it
 		 * now and wake the check task up to be sure the result is
 		 * handled ASAP. */
 		conn_sock_drain(conn);
 		cs_close(cs);
 		ret = -1;
-
-		if (check->wait_list.events)
-			cs->conn->mux->unsubscribe(cs, check->wait_list.events, &check->wait_list);
-
 		/* We may have been scheduled to run, and the
 		 * I/O handler expects to have a cs, so remove
 		 * the tasklet
@@ -864,7 +860,7 @@ static struct task *process_chk_conn(struct task *t, void *context, unsigned sho
 		 * is disabled.
 		 */
 		if (((check->state & (CHK_ST_ENABLED | CHK_ST_PAUSED)) != CHK_ST_ENABLED) ||
-		    proxy->state == PR_STSTOPPED)
+		    proxy->disabled)
 			goto reschedule;
 
 		/* we'll initiate a new check */
@@ -887,18 +883,8 @@ static struct task *process_chk_conn(struct task *t, void *context, unsigned sho
 			if ((conn->flags & CO_FL_ERROR) || cs->flags & CS_FL_ERROR || expired) {
 				chk_report_conn_err(check, 0, expired);
 			}
-			else {
-				if (check->state & CHK_ST_CLOSE_CONN) {
-					cs_destroy(cs);
-					cs = NULL;
-					conn = NULL;
-					check->cs = NULL;
-					check->state &= ~CHK_ST_CLOSE_CONN;
-					tcpcheck_main(check);
-				}
-				if (check->result == CHK_RES_UNKNOWN)
-					goto out_unlock; /* timeout not reached, wait again */
-			}
+			else
+				goto out_unlock; /* timeout not reached, wait again */
 		}
 
 		/* check complete or aborted */
@@ -1292,7 +1278,7 @@ static int init_srv_check(struct server *srv)
 	int ret = 0;
 	int check_type;
 
-	if (!srv->do_check || !(srv->proxy->cap & PR_CAP_BE))
+	if (!srv->do_check)
 		goto out;
 
 	check_type = srv->check.tcpcheck_rules->flags & TCPCHK_RULES_PROTO_CHK;
@@ -1417,7 +1403,7 @@ static int init_srv_agent_check(struct server *srv)
 	const char *err;
 	int ret = 0;
 
-	if (!srv->do_agent || !(srv->proxy->cap & PR_CAP_BE))
+	if (!srv->do_agent)
 		goto out;
 
 	/* If there is no connect rule preceding all send / expect rules, an
@@ -2714,7 +2700,6 @@ static int srv_parse_addr(char **args, int *cur_arg, struct proxy *curpx, struct
 			  char **errmsg)
 {
 	struct sockaddr_storage *sk;
-	struct protocol *proto;
 	int port1, port2, err_code = 0;
 
 
@@ -2723,25 +2708,13 @@ static int srv_parse_addr(char **args, int *cur_arg, struct proxy *curpx, struct
 		goto error;
 	}
 
-	sk = str2sa_range(args[*cur_arg+1], NULL, &port1, &port2, errmsg, NULL, NULL, 1);
+	sk = str2sa_range(args[*cur_arg+1], NULL, &port1, &port2, NULL, NULL, errmsg, NULL, NULL,
+	                  PA_O_RESOLVE | PA_O_PORT_OK | PA_O_STREAM | PA_O_CONNECT);
 	if (!sk) {
 		memprintf(errmsg, "'%s' : %s", args[*cur_arg], *errmsg);
 		goto error;
 	}
 
-	proto = protocol_by_family(sk->ss_family);
-	if (!proto || !proto->connect) {
-		memprintf(errmsg, "'%s %s' : connect() not supported for this address family.",
-		          args[*cur_arg], args[*cur_arg+1]);
-		goto error;
-	}
-
-	if (port1 != port2) {
-		memprintf(errmsg, "'%s' : port ranges and offsets are not allowed in '%s'.",
-		          args[*cur_arg], args[*cur_arg+1]);
-		goto error;
-	}
-
 	srv->check.addr = srv->agent.addr = *sk;
 	srv->flags |= SRV_F_CHECKADDR;
 	srv->flags |= SRV_F_AGENTADDR;
@@ -2790,12 +2763,6 @@ static int srv_parse_agent_check(char **args, int *cur_arg, struct proxy *curpx,
 	if (srv->do_agent)
 		goto out;
 
-	if (!(curpx->cap & PR_CAP_BE)) {
-		memprintf(errmsg, "'%s' ignored because %s '%s' has no backend capability",
-			  args[*cur_arg], proxy_type_str(curpx), curpx->id);
-		return ERR_WARN;
-	}
-
 	if (!rules) {
 		rules = calloc(1, sizeof(*rules));
 		if (!rules) {
@@ -2995,12 +2962,6 @@ static int srv_parse_no_agent_check(char **args, int *cur_arg, struct proxy *cur
 static int srv_parse_check(char **args, int *cur_arg, struct proxy *curpx, struct server *srv,
 			   char **errmsg)
 {
-	if (!(curpx->cap & PR_CAP_BE)) {
-		memprintf(errmsg, "'%s' ignored because %s '%s' has no backend capability",
-			  args[*cur_arg], proxy_type_str(curpx), curpx->id);
-		return ERR_WARN;
-	}
-
 	srv->do_check = 1;
 	return 0;
 }
