@@ -291,7 +291,7 @@ flt_init_all()
 	int err_code = 0;
 
 	for (px = proxies_list; px; px = px->next) {
-		if (px->state == PR_STSTOPPED) {
+		if (px->disabled) {
 			flt_deinit(px);
 			continue;
 		}
@@ -314,7 +314,7 @@ flt_init_all_per_thread()
 	int err_code = 0;
 
 	for (px = proxies_list; px; px = px->next) {
-		if (px->state == PR_STSTOPPED)
+		if (px->disabled)
 			continue;
 
 		err_code = flt_init_per_thread(px);
@@ -473,8 +473,6 @@ flt_stream_start(struct stream *s)
 		if (FLT_OPS(filter)->stream_start && FLT_OPS(filter)->stream_start(s, filter) < 0)
 			return -1;
 	}
-	if (strm_li(s) && (strm_li(s)->analysers & AN_REQ_FLT_START_FE))
-		s->req.flags |= CF_FLT_ANALYZE;
 	return 0;
 }
 
@@ -533,10 +531,6 @@ flt_set_stream_backend(struct stream *s, struct proxy *be)
 		    FLT_OPS(filter)->stream_set_backend(s, filter, be) < 0)
 			return -1;
 	}
-	if (be->be_req_ana & AN_REQ_FLT_START_BE)
-		s->req.flags |= CF_FLT_ANALYZE;
-	if ((strm_fe(s)->fe_rsp_ana | be->be_rsp_ana) & (AN_RES_FLT_START_FE|AN_RES_FLT_START_BE))
-		s->res.flags |= CF_FLT_ANALYZE;
 
 	return 0;
 }
@@ -638,8 +632,6 @@ flt_http_payload(struct stream *s, struct http_msg *msg, unsigned int len)
 	unsigned int out = co_data(msg->chn);
 	int ret, data;
 
-	strm_flt(s)->flags &= ~STRM_FLT_FL_HOLD_HTTP_HDRS;
-
 	ret = data = len - out;
 	DBG_TRACE_ENTER(STRM_EV_STRM_ANA|STRM_EV_HTTP_ANA|STRM_EV_FLT_ANA, s, s->txn, msg);
 	list_for_each_entry(filter, &strm_flt(s)->filters, list) {
@@ -664,22 +656,11 @@ flt_http_payload(struct stream *s, struct http_msg *msg, unsigned int len)
 		}
 	}
 
-	/* If nothing was forwarded yet, we take care to hold the headers if
-	 * following conditions are met :
-	 *
-	 *  - *strm_off == 0 (nothing forwarded yet)
-	 *  - ret == 0       (no data forwarded at all on this turn)
-	 *  - STRM_FLT_FL_HOLD_HTTP_HDRS flag set (at least one filter want to hold the headers)
-	 *
-	 * Be careful, STRM_FLT_FL_HOLD_HTTP_HDRS is removed before each http_payload loop.
-	 * Thus, it must explicitly be set when necessary. We must do that to hold the headers
-	 * when there is no payload.
-	 */
-	if (!ret && !*strm_off && (strm_flt(s)->flags & STRM_FLT_FL_HOLD_HTTP_HDRS))
-		goto end;
-
-	ret = data;
-	*strm_off += ret;
+	/* Only forward data if the last filter decides to forward something */
+	if (ret > 0) {
+		ret = data;
+		*strm_off += ret;
+	}
  end:
 	DBG_TRACE_LEAVE(STRM_EV_STRM_ANA|STRM_EV_HTTP_ANA|STRM_EV_FLT_ANA, s);
 	return ret;
@@ -757,7 +738,6 @@ flt_pre_analyze(struct stream *s, struct channel *chn, unsigned int an_bit)
 			ret = FLT_OPS(filter)->channel_pre_analyze(s, filter, chn, an_bit);
 			if (ret <= 0)
 				BREAK_EXECUTION(s, chn, check_result);
-			filter->pre_analyzers &= ~an_bit;
 		}
 	} RESUME_FILTER_END;
 
@@ -790,7 +770,6 @@ flt_post_analyze(struct stream *s, struct channel *chn, unsigned int an_bit)
 			ret = FLT_OPS(filter)->channel_post_analyze(s, filter, chn, an_bit);
 			if (ret < 0)
 				break;
-			filter->post_analyzers &= ~an_bit;
 		}
 	}
 	ret = handle_analyzer_result(s, chn, 0, ret);
