@@ -37,7 +37,7 @@ static inline void fwlc_dequeue_srv(struct server *s)
 	eb32_delete(&s->lb_node);
 }
 
-/* Queue a server in its associated tree, assuming the <eweight> is >0.
+/* Queue a server in its associated tree, assuming the weight is >0.
  * Servers are sorted by (#conns+1)/weight. To ensure maximum accuracy,
  * we use (#conns+1)*SRV_EWGHT_MAX/eweight as the sorting key. The reason
  * for using #conns+1 is to sort by weights in case the server is picked
@@ -47,17 +47,13 @@ static inline void fwlc_dequeue_srv(struct server *s)
  * connection are always picked first so that under low loads, it's not
  * always the single server with the highest weight that gets picked.
  *
- * NOTE: Depending on the calling context, we use s->next_eweight or
- *       s->cur_eweight. The next value is used when the server state is updated
- *       (because the weight changed for instance). During this step, the server
- *       state is not yet committed. The current value is used to reposition the
- *       server in the tree. This happens when the server is used.
- *
  * The server's lock and the lbprm's lock must be held.
  */
-static inline void fwlc_queue_srv(struct server *s, unsigned int eweight)
+static inline void fwlc_queue_srv(struct server *s)
 {
-	s->lb_node.key = s->served ? (s->served + 1) * SRV_EWGHT_MAX / eweight : 0;
+	unsigned int inflight = s->served + s->nbpend;
+
+	s->lb_node.key = inflight ? (inflight + 1) * SRV_EWGHT_MAX / s->next_eweight : 0;
 	eb32_insert(s->lb_tree, &s->lb_node);
 }
 
@@ -69,12 +65,12 @@ static inline void fwlc_queue_srv(struct server *s, unsigned int eweight)
  */
 static void fwlc_srv_reposition(struct server *s)
 {
-	HA_SPIN_LOCK(LBPRM_LOCK, &s->proxy->lbprm.lock);
+	HA_RWLOCK_WRLOCK(LBPRM_LOCK, &s->proxy->lbprm.lock);
 	if (s->lb_tree) {
 		fwlc_dequeue_srv(s);
-		fwlc_queue_srv(s, s->cur_eweight);
+		fwlc_queue_srv(s);
 	}
-	HA_SPIN_UNLOCK(LBPRM_LOCK, &s->proxy->lbprm.lock);
+	HA_RWLOCK_WRUNLOCK(LBPRM_LOCK, &s->proxy->lbprm.lock);
 }
 
 /* This function updates the server trees according to server <srv>'s new
@@ -94,7 +90,7 @@ static void fwlc_set_server_status_down(struct server *srv)
 
 	if (srv_willbe_usable(srv))
 		goto out_update_state;
-	HA_SPIN_LOCK(LBPRM_LOCK, &p->lbprm.lock);
+	HA_RWLOCK_WRLOCK(LBPRM_LOCK, &p->lbprm.lock);
 
 
 	if (!srv_currently_usable(srv))
@@ -128,7 +124,7 @@ static void fwlc_set_server_status_down(struct server *srv)
 out_update_backend:
 	/* check/update tot_used, tot_weight */
 	update_backend_weight(p);
-	HA_SPIN_UNLOCK(LBPRM_LOCK, &p->lbprm.lock);
+	HA_RWLOCK_WRUNLOCK(LBPRM_LOCK, &p->lbprm.lock);
 
  out_update_state:
 	srv_lb_commit_status(srv);
@@ -153,7 +149,7 @@ static void fwlc_set_server_status_up(struct server *srv)
 	if (!srv_willbe_usable(srv))
 		goto out_update_state;
 
-	HA_SPIN_LOCK(LBPRM_LOCK, &p->lbprm.lock);
+	HA_RWLOCK_WRLOCK(LBPRM_LOCK, &p->lbprm.lock);
 
 	if (srv_currently_usable(srv))
 		/* server was already up */
@@ -187,12 +183,12 @@ static void fwlc_set_server_status_up(struct server *srv)
 	}
 
 	/* note that eweight cannot be 0 here */
-	fwlc_queue_srv(srv, srv->next_eweight);
+	fwlc_queue_srv(srv);
 
  out_update_backend:
 	/* check/update tot_used, tot_weight */
 	update_backend_weight(p);
-	HA_SPIN_UNLOCK(LBPRM_LOCK, &p->lbprm.lock);
+	HA_RWLOCK_WRUNLOCK(LBPRM_LOCK, &p->lbprm.lock);
 
  out_update_state:
 	srv_lb_commit_status(srv);
@@ -235,7 +231,7 @@ static void fwlc_update_server_weight(struct server *srv)
 		return;
 	}
 
-	HA_SPIN_LOCK(LBPRM_LOCK, &p->lbprm.lock);
+	HA_RWLOCK_WRLOCK(LBPRM_LOCK, &p->lbprm.lock);
 
 	if (srv->lb_tree)
 		fwlc_dequeue_srv(srv);
@@ -248,10 +244,10 @@ static void fwlc_update_server_weight(struct server *srv)
 		srv->lb_tree = &p->lbprm.fwlc.act;
 	}
 
-	fwlc_queue_srv(srv, srv->next_eweight);
+	fwlc_queue_srv(srv);
 
 	update_backend_weight(p);
-	HA_SPIN_UNLOCK(LBPRM_LOCK, &p->lbprm.lock);
+	HA_RWLOCK_WRUNLOCK(LBPRM_LOCK, &p->lbprm.lock);
 
 	srv_lb_commit_status(srv);
 }
@@ -288,7 +284,7 @@ void fwlc_init_server_tree(struct proxy *p)
 		if (!srv_currently_usable(srv))
 			continue;
 		srv->lb_tree = (srv->flags & SRV_F_BACKUP) ? &p->lbprm.fwlc.bck : &p->lbprm.fwlc.act;
-		fwlc_queue_srv(srv, srv->next_eweight);
+		fwlc_queue_srv(srv);
 	}
 }
 
@@ -304,7 +300,7 @@ struct server *fwlc_get_next_server(struct proxy *p, struct server *srvtoavoid)
 
 	srv = avoided = NULL;
 
-	HA_SPIN_LOCK(LBPRM_LOCK, &p->lbprm.lock);
+	HA_RWLOCK_RDLOCK(LBPRM_LOCK, &p->lbprm.lock);
 	if (p->srv_act)
 		node = eb32_first(&p->lbprm.fwlc.act);
 	else if (p->lbprm.fbck) {
@@ -327,7 +323,7 @@ struct server *fwlc_get_next_server(struct proxy *p, struct server *srvtoavoid)
 		struct server *s;
 
 		s = eb32_entry(node, struct server, lb_node);
-		if (!s->maxconn || (!s->nbpend && s->served < srv_dynamic_maxconn(s))) {
+		if (!s->maxconn || s->served + s->nbpend < srv_dynamic_maxconn(s) + s->maxqueue) {
 			if (s != srvtoavoid) {
 				srv = s;
 				break;
@@ -340,7 +336,7 @@ struct server *fwlc_get_next_server(struct proxy *p, struct server *srvtoavoid)
 	if (!srv)
 		srv = avoided;
  out:
-	HA_SPIN_UNLOCK(LBPRM_LOCK, &p->lbprm.lock);
+	HA_RWLOCK_RDUNLOCK(LBPRM_LOCK, &p->lbprm.lock);
 	return srv;
 }
 
