@@ -53,10 +53,8 @@ static void mux_pt_destroy(struct mux_pt_ctx *ctx)
 	}
 }
 
-/* Callback, used when we get I/Os while in idle mode. This one is exported so
- * that "show fd" can resolve it.
- */
-struct task *mux_pt_io_cb(struct task *t, void *tctx, unsigned short status)
+/* Callback, used when we get I/Os while in idle mode */
+static struct task *mux_pt_io_cb(struct task *t, void *tctx, unsigned short status)
 {
 	struct mux_pt_ctx *ctx = tctx;
 
@@ -110,7 +108,7 @@ static int mux_pt_init(struct connection *conn, struct proxy *prx, struct sessio
 	ctx->conn = conn;
 
 	if (!cs) {
-		cs = cs_new(conn);
+		cs = cs_new(conn, conn->target);
 		if (!cs)
 			goto fail_free_ctx;
 
@@ -177,7 +175,7 @@ static struct conn_stream *mux_pt_attach(struct connection *conn, struct session
 
 	if (ctx->wait_event.events)
 		conn->xprt->unsubscribe(ctx->conn, conn->xprt_ctx, SUB_RETRY_RECV, &ctx->wait_event);
-	cs = cs_new(conn);
+	cs = cs_new(conn, conn->target);
 	if (!cs)
 		goto fail;
 
@@ -218,7 +216,7 @@ static void mux_pt_detach(struct conn_stream *cs)
 	struct mux_pt_ctx *ctx = cs->conn->ctx;
 
 	/* Subscribe, to know if we got disconnected */
-	if (!conn_is_back(conn) && conn->owner != NULL &&
+	if (conn->owner != NULL &&
 	    !(conn->flags & (CO_FL_ERROR | CO_FL_SOCK_RD_SH | CO_FL_SOCK_WR_SH))) {
 		ctx->cs = NULL;
 		conn->xprt->subscribe(conn, conn->xprt_ctx, SUB_RETRY_RECV, &ctx->wait_event);
