@@ -10,8 +10,6 @@
  *
  */
 
-#define _GNU_SOURCE
-
 #include <errno.h>
 #include <fcntl.h>
 #include <signal.h>
@@ -129,13 +127,13 @@ void mworker_proc_list_to_env()
 /*
  * unserialize the proc list from the environment
  */
-int mworker_env_to_proc_list()
+void mworker_env_to_proc_list()
 {
 	char *msg, *token = NULL, *s1;
 
 	msg = getenv("HAPROXY_PROCESSES");
 	if (!msg)
-		return 0;
+		return;
 
 	while ((token = strtok_r(msg, "|", &s1))) {
 		struct mworker_proc *child;
@@ -145,10 +143,6 @@ int mworker_env_to_proc_list()
 		msg = NULL;
 
 		child = calloc(1, sizeof(*child));
-		if (!child) {
-			ha_alert("Out of memory while trying to allocate a worker process structure.");
-			return -1;
-		}
 
 		while ((subtoken = strtok_r(token, ";", &s2))) {
 
@@ -195,8 +189,6 @@ int mworker_env_to_proc_list()
 	}
 
 	unsetenv("HAPROXY_PROCESSES");
-
-	return 0;
 }
 
 /* Signal blocking and unblocking */
@@ -342,7 +334,8 @@ restart_wait:
 /* This wrapper is called from the workers. It is registered instead of the
  * normal listener_accept() so the worker can exit() when it detects that the
  * master closed the IPC FD. If it's not a close, we just call the regular
- * listener_accept() function */
+ * listener_accept() function.
+ */
 void mworker_accept_wrapper(int fd)
 {
 	char c;
@@ -359,7 +352,10 @@ void mworker_accept_wrapper(int fd)
 			}
 			break;
 		} else if (ret > 0) {
-			listener_accept(fd);
+			struct listener *l = fdtab[fd].owner;
+
+			if (l)
+				listener_accept(l);
 			return;
 		} else if (ret == 0) {
 			/* At this step the master is down before
@@ -430,14 +426,8 @@ void mworker_cleanlisteners()
 
 		list_for_each_entry_safe(l, l_next, &curproxy->conf.listeners, by_fe) {
 			/* remove the listener, but not those we need in the master... */
-			if (!(l->options & LI_O_MWORKER)) {
-				/* unbind the listener but does not close if
-				   the FD is inherited with fd@ from the parent
-				   process */
-				if (l->options & LI_O_INHERITED)
-					unbind_listener_no_close(l);
-				else
-					unbind_listener(l);
+			if (!(l->rx.flags & RX_F_MWORKER)) {
+				unbind_listener(l);
 				delete_listener(l);
 			} else {
 				listen_in_master = 1;
@@ -445,36 +435,10 @@ void mworker_cleanlisteners()
 		}
 		/* if the proxy shouldn't be in the master, we stop it */
 		if (!listen_in_master)
-			curproxy->state = PR_STSTOPPED;
+			curproxy->disabled = 1;
 	}
 }
 
-/* Upon a configuration loading error some mworker_proc and FDs/server were
- * assigned but the worker was never forked, we must close the FDs and
- * remove the server
- */
-void mworker_cleanup_proc()
-{
-	struct mworker_proc *child, *it;
-
-	list_for_each_entry_safe(child, it, &proc_list, list) {
-
-		if (child->pid == -1) {
-			/* Close the socketpair master side.  We don't need to
-			 * close the worker side, because it's stored in the
-			 * GLOBAL cli listener which was supposed to be in the
-			 * worker and which will be closed in
-			 * mworker_cleanlisteners()
-			 */
-			if (child->ipc_fd[0] > -1)
-				close(child->ipc_fd[0]);
-			LIST_DEL(&child->list);
-			mworker_free_child(child);
-		}
-	}
-}
-
-
 /*  Displays workers and processes  */
 static int cli_io_handler_show_proc(struct appctx *appctx)
 {
