@@ -35,6 +35,7 @@
 #include <haproxy/obj_type-t.h>
 #include <haproxy/peers.h>
 #include <haproxy/pool.h>
+#include <haproxy/protocol.h>
 #include <haproxy/proto_tcp.h>
 #include <haproxy/proxy.h>
 #include <haproxy/server-t.h>
@@ -137,8 +138,6 @@ const char *proxy_mode_str(int mode) {
 		return "tcp";
 	else if (mode == PR_MODE_HTTP)
 		return "http";
-	else if (mode == PR_MODE_HEALTH)
-		return "health";
 	else if (mode == PR_MODE_CLI)
 		return "cli";
 	else
@@ -570,6 +569,165 @@ proxy_parse_retry_on(char **args, int section, struct proxy *curpx,
 	return 0;
 }
 
+#ifdef TCP_KEEPCNT
+/* This function parses "{cli|srv}tcpka-cnt" statements */
+static int proxy_parse_tcpka_cnt(char **args, int section, struct proxy *proxy,
+                                    struct proxy *defpx, const char *file, int line,
+                                    char **err)
+{
+	int retval;
+	char *res;
+	unsigned int tcpka_cnt;
+
+	retval = 0;
+
+	if (*args[1] == 0) {
+		memprintf(err, "'%s' expects an integer value", args[0]);
+		return -1;
+	}
+
+	tcpka_cnt = strtol(args[1], &res, 0);
+	if (*res) {
+		memprintf(err, "'%s' : unexpected character '%c' in integer value '%s'", args[0], *res, args[1]);
+		return -1;
+	}
+
+	if (!strcmp(args[0], "clitcpka-cnt")) {
+		if (!(proxy->cap & PR_CAP_FE)) {
+			memprintf(err, "%s will be ignored because %s '%s' has no frontend capability",
+			          args[0], proxy_type_str(proxy), proxy->id);
+			retval = 1;
+		}
+		proxy->clitcpka_cnt = tcpka_cnt;
+	} else if (!strcmp(args[0], "srvtcpka-cnt")) {
+		if (!(proxy->cap & PR_CAP_BE)) {
+			memprintf(err, "%s will be ignored because %s '%s' has no backend capability",
+			          args[0], proxy_type_str(proxy), proxy->id);
+			retval = 1;
+		}
+		proxy->srvtcpka_cnt = tcpka_cnt;
+	} else {
+		/* unreachable */
+		memprintf(err, "'%s': unknown keyword", args[0]);
+		return -1;
+	}
+
+	return retval;
+}
+#endif
+
+#ifdef TCP_KEEPIDLE
+/* This function parses "{cli|srv}tcpka-idle" statements */
+static int proxy_parse_tcpka_idle(char **args, int section, struct proxy *proxy,
+                                  struct proxy *defpx, const char *file, int line,
+                                  char **err)
+{
+	int retval;
+	const char *res;
+	unsigned int tcpka_idle;
+
+	retval = 0;
+
+	if (*args[1] == 0) {
+		memprintf(err, "'%s' expects an integer value", args[0]);
+		return -1;
+	}
+	res = parse_time_err(args[1], &tcpka_idle, TIME_UNIT_S);
+	if (res == PARSE_TIME_OVER) {
+		memprintf(err, "timer overflow in argument '%s' to '%s' (maximum value is 2147483647 ms or ~24.8 days)",
+			  args[1], args[0]);
+		return -1;
+	}
+	else if (res == PARSE_TIME_UNDER) {
+		memprintf(err, "timer underflow in argument '%s' to '%s' (minimum non-null value is 1 ms)",
+			  args[1], args[0]);
+		return -1;
+	}
+	else if (res) {
+		memprintf(err, "unexpected character '%c' in argument to <%s>.\n", *res, args[0]);
+		return -1;
+	}
+
+	if (!strcmp(args[0], "clitcpka-idle")) {
+		if (!(proxy->cap & PR_CAP_FE)) {
+			memprintf(err, "%s will be ignored because %s '%s' has no frontend capability",
+			          args[0], proxy_type_str(proxy), proxy->id);
+			retval = 1;
+		}
+		proxy->clitcpka_idle = tcpka_idle;
+	} else if (!strcmp(args[0], "srvtcpka-idle")) {
+		if (!(proxy->cap & PR_CAP_BE)) {
+			memprintf(err, "%s will be ignored because %s '%s' has no backend capability",
+			          args[0], proxy_type_str(proxy), proxy->id);
+			retval = 1;
+		}
+		proxy->srvtcpka_idle = tcpka_idle;
+	} else {
+		/* unreachable */
+		memprintf(err, "'%s': unknown keyword", args[0]);
+		return -1;
+	}
+
+	return retval;
+}
+#endif
+
+#ifdef TCP_KEEPINTVL
+/* This function parses "{cli|srv}tcpka-intvl" statements */
+static int proxy_parse_tcpka_intvl(char **args, int section, struct proxy *proxy,
+		                   struct proxy *defpx, const char *file, int line,
+                                   char **err)
+{
+	int retval;
+	const char *res;
+	unsigned int tcpka_intvl;
+
+	retval = 0;
+
+	if (*args[1] == 0) {
+		memprintf(err, "'%s' expects an integer value", args[0]);
+		return -1;
+	}
+	res = parse_time_err(args[1], &tcpka_intvl, TIME_UNIT_S);
+	if (res == PARSE_TIME_OVER) {
+		memprintf(err, "timer overflow in argument '%s' to '%s' (maximum value is 2147483647 ms or ~24.8 days)",
+			  args[1], args[0]);
+		return -1;
+	}
+	else if (res == PARSE_TIME_UNDER) {
+		memprintf(err, "timer underflow in argument '%s' to '%s' (minimum non-null value is 1 ms)",
+			  args[1], args[0]);
+		return -1;
+	}
+	else if (res) {
+		memprintf(err, "unexpected character '%c' in argument to <%s>.\n", *res, args[0]);
+		return -1;
+	}
+
+	if (!strcmp(args[0], "clitcpka-intvl")) {
+		if (!(proxy->cap & PR_CAP_FE)) {
+			memprintf(err, "%s will be ignored because %s '%s' has no frontend capability",
+			          args[0], proxy_type_str(proxy), proxy->id);
+			retval = 1;
+		}
+		proxy->clitcpka_intvl = tcpka_intvl;
+	} else if (!strcmp(args[0], "srvtcpka-intvl")) {
+		if (!(proxy->cap & PR_CAP_BE)) {
+			memprintf(err, "%s will be ignored because %s '%s' has no backend capability",
+			          args[0], proxy_type_str(proxy), proxy->id);
+			retval = 1;
+		}
+		proxy->srvtcpka_intvl = tcpka_intvl;
+	} else {
+		/* unreachable */
+		memprintf(err, "'%s': unknown keyword", args[0]);
+		return -1;
+	}
+
+	return retval;
+}
+#endif
+
 /* This function inserts proxy <px> into the tree of known proxies. The proxy's
  * name is used as the storing key so it must already have been initialized.
  */
@@ -883,72 +1041,49 @@ void init_new_proxy(struct proxy *p)
 	/* Default to only allow L4 retries */
 	p->retry_type = PR_RE_CONN_FAILED;
 
-	HA_SPIN_INIT(&p->lock);
+	HA_RWLOCK_INIT(&p->lock);
 }
 
-/*
- * This function creates all proxy sockets. It should be done very early,
- * typically before privileges are dropped. The sockets will be registered
- * but not added to any fd_set, in order not to loose them across the fork().
- * The proxies also start in READY state because they all have their listeners
- * bound.
- *
- * Its return value is composed from ERR_NONE, ERR_RETRYABLE and ERR_FATAL.
- * Retryable errors will only be printed if <verbose> is not zero.
+/* to be called under the proxy lock after stopping some listeners. This will
+ * automatically update the p->disabled flag after stopping the last one, and
+ * will emit a log indicating the proxy's condition. The function is idempotent
+ * so that it will not emit multiple logs; a proxy will be disabled only once.
  */
-int start_proxies(int verbose)
+void proxy_cond_disable(struct proxy *p)
 {
-	struct proxy *curproxy;
-	struct listener *listener;
-	int lerr, err = ERR_NONE;
-	int pxerr;
-	char msg[100];
+	if (p->disabled)
+		return;
 
-	for (curproxy = proxies_list; curproxy != NULL; curproxy = curproxy->next) {
-		if (curproxy->state != PR_STNEW)
-			continue; /* already initialized */
+	if (p->li_ready + p->li_paused > 0)
+		return;
 
-		pxerr = 0;
-		list_for_each_entry(listener, &curproxy->conf.listeners, by_fe) {
-			if (listener->state != LI_ASSIGNED)
-				continue; /* already started */
+	p->disabled = 1;
 
-			lerr = listener->proto->bind(listener, msg, sizeof(msg));
+	if (!(proc_mask(p->bind_proc) & pid_bit))
+		goto silent;
 
-			/* errors are reported if <verbose> is set or if they are fatal */
-			if (verbose || (lerr & (ERR_FATAL | ERR_ABORT))) {
-				if (lerr & ERR_ALERT)
-					ha_alert("Starting %s %s: %s\n",
-						 proxy_type_str(curproxy), curproxy->id, msg);
-				else if (lerr & ERR_WARN)
-					ha_warning("Starting %s %s: %s\n",
-						   proxy_type_str(curproxy), curproxy->id, msg);
-			}
+	/* Note: syslog proxies use their own loggers so while it's somewhat OK
+	 * to report them being stopped as a warning, we must not spam their log
+	 * servers which are in fact production servers. For other types (CLI,
+	 * peers, etc) we must not report them at all as they're not really on
+	 * the data plane but on the control plane.
+	 */
+	if (p->mode == PR_MODE_TCP || p->mode == PR_MODE_HTTP || p->mode == PR_MODE_SYSLOG)
+		ha_warning("Proxy %s stopped (cumulated conns: FE: %lld, BE: %lld).\n",
+			   p->id, p->fe_counters.cum_conn, p->be_counters.cum_conn);
 
-			err |= lerr;
-			if (lerr & (ERR_ABORT | ERR_FATAL)) {
-				pxerr |= 1;
-				break;
-			}
-			else if (lerr & ERR_CODE) {
-				pxerr |= 1;
-				continue;
-			}
-		}
+	if (p->mode == PR_MODE_TCP || p->mode == PR_MODE_HTTP)
+		send_log(p, LOG_WARNING, "Proxy %s stopped (cumulated conns: FE: %lld, BE: %lld).\n",
+			 p->id, p->fe_counters.cum_conn, p->be_counters.cum_conn);
 
-		if (!pxerr) {
-			curproxy->state = PR_STREADY;
-			send_log(curproxy, LOG_NOTICE, "Proxy %s started.\n", curproxy->id);
-		}
+ silent:
+	if (p->table && p->table->size && p->table->sync_task)
+		task_wakeup(p->table->sync_task, TASK_WOKEN_MSG);
 
-		if (err & ERR_ABORT)
-			break;
-	}
-
-	return err;
+	if (p->task)
+		task_wakeup(p->task, TASK_WOKEN_MSG);
 }
 
-
 /*
  * This is the proxy management task. It enables proxies when there are enough
  * free streams, or stops them when the table is full. It is designed to be
@@ -966,14 +1101,10 @@ struct task *manage_proxy(struct task *t, void *context, unsigned short state)
 	 */
 
 	/* first, let's check if we need to stop the proxy */
-	if (unlikely(stopping && p->state != PR_STSTOPPED)) {
+	if (unlikely(stopping && !p->disabled)) {
 		int t;
 		t = tick_remain(now_ms, p->stop_time);
 		if (t == 0) {
-			ha_warning("Proxy %s stopped (cumulated conns: FE: %lld, BE: %lld).\n",
-				   p->id, p->fe_counters.cum_conn, p->be_counters.cum_conn);
-			send_log(p, LOG_WARNING, "Proxy %s stopped (cumulated conns: FE: %lld, BE: %lld).\n",
-				 p->id, p->fe_counters.cum_conn, p->be_counters.cum_conn);
 			stop_proxy(p);
 			/* try to free more memory */
 			pool_gc(NULL);
@@ -990,7 +1121,7 @@ struct task *manage_proxy(struct task *t, void *context, unsigned short state)
 	 * be in neither list. Any entry being dumped will have ref_cnt > 0.
 	 * However we protect tables that are being synced to peers.
 	 */
-	if (unlikely(stopping && p->state == PR_STSTOPPED && p->table && p->table->current)) {
+	if (unlikely(stopping && p->disabled && p->table && p->table->current)) {
 		if (!p->table->syncing) {
 			stktable_trash_oldest(p->table, p->table->current);
 			pool_gc(NULL);
@@ -1006,15 +1137,8 @@ struct task *manage_proxy(struct task *t, void *context, unsigned short state)
 		goto out;
 
 	/* check the various reasons we may find to block the frontend */
-	if (unlikely(p->feconn >= p->maxconn)) {
-		if (p->state == PR_STREADY)
-			p->state = PR_STFULL;
+	if (unlikely(p->feconn >= p->maxconn))
 		goto out;
-	}
-
-	/* OK we have no reason to block, so let's unblock if we were blocking */
-	if (p->state == PR_STFULL)
-		p->state = PR_STREADY;
 
 	if (p->fe_sps_lim &&
 	    (wait = next_event_delay(&p->fe_sess_per_sec, p->fe_sps_lim, 0))) {
@@ -1106,7 +1230,6 @@ struct task *hard_stop(struct task *t, void *context, unsigned short state)
 void soft_stop(void)
 {
 	struct proxy *p;
-	struct peers *prs;
 	struct task *task;
 
 	stopping = 1;
@@ -1122,23 +1245,16 @@ void soft_stop(void)
 			ha_alert("out of memory trying to allocate the hard-stop task.\n");
 		}
 	}
+
+	/* stop all stoppable listeners, resulting in disabling all proxies
+	 * that don't use a grace period.
+	 */
+	protocol_stop_now();
+
 	p = proxies_list;
 	tv_update_date(0,1); /* else, the old time before select will be used */
 	while (p) {
-		/* Zombie proxy, let's close the file descriptors */
-		if (p->state == PR_STSTOPPED &&
-		    !LIST_ISEMPTY(&p->conf.listeners) &&
-		    LIST_ELEM(p->conf.listeners.n,
-		    struct listener *, by_fe)->state > LI_ASSIGNED) {
-			struct listener *l;
-			list_for_each_entry(l, &p->conf.listeners, by_fe) {
-				if (l->state > LI_ASSIGNED)
-					close(l->fd);
-				l->state = LI_INIT;
-			}
-		}
-
-		if (p->state != PR_STSTOPPED) {
+		if (!p->disabled) {
 			ha_warning("Stopping %s %s in %d ms.\n", proxy_cap_str(p->cap), p->id, p->grace);
 			send_log(p, LOG_WARNING, "Stopping %s %s in %d ms.\n", proxy_cap_str(p->cap), p->id, p->grace);
 			p->stop_time = tick_add(now_ms, p->grace);
@@ -1155,87 +1271,33 @@ void soft_stop(void)
 		p = p->next;
 	}
 
-	prs = cfg_peers;
-	while (prs) {
-		if (prs->peers_fe)
-			stop_proxy(prs->peers_fe);
-		prs = prs->next;
-	}
 	/* signal zero is used to broadcast the "stopping" event */
 	signal_handler(0);
 }
 
 
 /* Temporarily disables listening on all of the proxy's listeners. Upon
- * success, the proxy enters the PR_PAUSED state. If disabling at least one
- * listener returns an error, then the proxy state is set to PR_STERROR
- * because we don't know how to resume from this. The function returns 0
+ * success, the proxy enters the PR_PAUSED state. The function returns 0
  * if it fails, or non-zero on success.
  */
 int pause_proxy(struct proxy *p)
 {
 	struct listener *l;
 
-	if (!(p->cap & PR_CAP_FE) || p->state == PR_STERROR ||
-	    p->state == PR_STSTOPPED || p->state == PR_STPAUSED)
+	if (!(p->cap & PR_CAP_FE) || p->disabled || !p->li_ready)
 		return 1;
 
-	ha_warning("Pausing %s %s.\n", proxy_cap_str(p->cap), p->id);
-	send_log(p, LOG_WARNING, "Pausing %s %s.\n", proxy_cap_str(p->cap), p->id);
+	list_for_each_entry(l, &p->conf.listeners, by_fe)
+		pause_listener(l);
 
-	list_for_each_entry(l, &p->conf.listeners, by_fe) {
-		if (!pause_listener(l))
-			p->state = PR_STERROR;
-	}
-
-	if (p->state == PR_STERROR) {
+	if (p->li_ready) {
 		ha_warning("%s %s failed to enter pause mode.\n", proxy_cap_str(p->cap), p->id);
 		send_log(p, LOG_WARNING, "%s %s failed to enter pause mode.\n", proxy_cap_str(p->cap), p->id);
 		return 0;
 	}
-
-	p->state = PR_STPAUSED;
 	return 1;
 }
 
-/* This function makes the proxy unusable, but keeps the listening sockets
- * opened, so that if any process requests them, we are able to serve them.
- * This should only be called early, before we started accepting requests.
- */
-void zombify_proxy(struct proxy *p)
-{
-	struct listener *l;
-	struct listener *first_to_listen = NULL;
-
-	list_for_each_entry(l, &p->conf.listeners, by_fe) {
-		enum li_state oldstate = l->state;
-
-		unbind_listener_no_close(l);
-		if (l->state >= LI_ASSIGNED) {
-			delete_listener(l);
-		}
-		/*
-		 * Pretend we're still up and running so that the fd
-		 * will be sent if asked.
-		 */
-		l->state = LI_ZOMBIE;
-		if (!first_to_listen && oldstate >= LI_LISTEN)
-			first_to_listen = l;
-	}
-	/* Quick hack : at stop time, to know we have to close the sockets
-	 * despite the proxy being marked as stopped, make the first listener
-	 * of the listener list an active one, so that we don't have to
-	 * parse the whole list to be sure.
-	 */
-	if (first_to_listen && LIST_ELEM(p->conf.listeners.n,
-	    struct listener *, by_fe) != first_to_listen) {
-		LIST_DEL(&l->by_fe);
-		LIST_ADD(&p->conf.listeners, &l->by_fe);
-	}
-
-	p->state = PR_STSTOPPED;
-}
-
 /*
  * This function completely stops a proxy and releases its listeners. It has
  * to be called when going down in order to release the ports so that another
@@ -1247,29 +1309,18 @@ void zombify_proxy(struct proxy *p)
 void stop_proxy(struct proxy *p)
 {
 	struct listener *l;
-	int nostop = 0;
 
-	HA_SPIN_LOCK(PROXY_LOCK, &p->lock);
+	HA_RWLOCK_WRLOCK(PROXY_LOCK, &p->lock);
 
-	list_for_each_entry(l, &p->conf.listeners, by_fe) {
-		if (l->options & LI_O_NOSTOP) {
-			HA_ATOMIC_ADD(&unstoppable_jobs, 1);
-			nostop = 1;
-			continue;
-		}
-		/* The master should not close an inherited FD */
-		if (master && (l->options & LI_O_INHERITED))
-			unbind_listener_no_close(l);
-		else
-			unbind_listener(l);
-		if (l->state >= LI_ASSIGNED) {
-			delete_listener(l);
-		}
+	list_for_each_entry(l, &p->conf.listeners, by_fe)
+		stop_listener(l, 1, 0, 0);
+
+	if (!p->disabled && !p->li_ready) {
+		/* might be just a backend */
+		p->disabled = 1;
 	}
-	if (!nostop)
-		p->state = PR_STSTOPPED;
 
-	HA_SPIN_UNLOCK(PROXY_LOCK, &p->lock);
+	HA_RWLOCK_WRUNLOCK(PROXY_LOCK, &p->lock);
 }
 
 /* This function resumes listening on the specified proxy. It scans all of its
@@ -1282,18 +1333,15 @@ int resume_proxy(struct proxy *p)
 	struct listener *l;
 	int fail;
 
-	if (p->state != PR_STPAUSED)
+	if (p->disabled || !p->li_paused)
 		return 1;
 
-	ha_warning("Enabling %s %s.\n", proxy_cap_str(p->cap), p->id);
-	send_log(p, LOG_WARNING, "Enabling %s %s.\n", proxy_cap_str(p->cap), p->id);
-
 	fail = 0;
 	list_for_each_entry(l, &p->conf.listeners, by_fe) {
 		if (!resume_listener(l)) {
 			int port;
 
-			port = get_host_port(&l->addr);
+			port = get_host_port(&l->rx.addr);
 			if (port) {
 				ha_warning("Port %d busy while trying to enable %s %s.\n",
 					   port, proxy_cap_str(p->cap), p->id);
@@ -1313,7 +1361,6 @@ int resume_proxy(struct proxy *p)
 		}
 	}
 
-	p->state = PR_STREADY;
 	if (fail) {
 		pause_proxy(p);
 		return 0;
@@ -1321,73 +1368,6 @@ int resume_proxy(struct proxy *p)
 	return 1;
 }
 
-/*
- * This function temporarily disables listening so that another new instance
- * can start listening. It is designed to be called upon reception of a
- * SIGTTOU, after which either a SIGUSR1 can be sent to completely stop
- * the proxy, or a SIGTTIN can be sent to listen again.
- */
-void pause_proxies(void)
-{
-	int err;
-	struct proxy *p;
-	struct peers *prs;
-
-	err = 0;
-	p = proxies_list;
-	tv_update_date(0,1); /* else, the old time before select will be used */
-	while (p) {
-		err |= !pause_proxy(p);
-		p = p->next;
-	}
-
-	prs = cfg_peers;
-	while (prs) {
-		if (prs->peers_fe)
-			err |= !pause_proxy(prs->peers_fe);
-		prs = prs->next;
-        }
-
-	if (err) {
-		ha_warning("Some proxies refused to pause, performing soft stop now.\n");
-		send_log(p, LOG_WARNING, "Some proxies refused to pause, performing soft stop now.\n");
-		soft_stop();
-	}
-}
-
-
-/*
- * This function reactivates listening. This can be used after a call to
- * sig_pause(), for example when a new instance has failed starting up.
- * It is designed to be called upon reception of a SIGTTIN.
- */
-void resume_proxies(void)
-{
-	int err;
-	struct proxy *p;
-	struct peers *prs;
-
-	err = 0;
-	p = proxies_list;
-	tv_update_date(0,1); /* else, the old time before select will be used */
-	while (p) {
-		err |= !resume_proxy(p);
-		p = p->next;
-	}
-
-	prs = cfg_peers;
-	while (prs) {
-		if (prs->peers_fe)
-			err |= !resume_proxy(prs->peers_fe);
-		prs = prs->next;
-        }
-
-	if (err) {
-		ha_warning("Some proxies refused to resume, a restart is probably needed to resume safe operations.\n");
-		send_log(p, LOG_WARNING, "Some proxies refused to resume, a restart is probably needed to resume safe operations.\n");
-	}
-}
-
 /* Set current stream's backend to <be>. Nothing is done if the
  * stream already had a backend assigned, which is indicated by
  * s->flags & SF_BE_ASSIGNED.
@@ -1579,14 +1559,14 @@ void proxy_capture_error(struct proxy *proxy, int is_back,
 	/* note: we still lock since we have to be certain that nobody is
 	 * dumping the output while we free.
 	 */
-	HA_SPIN_LOCK(PROXY_LOCK, &proxy->lock);
+	HA_RWLOCK_WRLOCK(PROXY_LOCK, &proxy->lock);
 	if (is_back) {
 		es = HA_ATOMIC_XCHG(&proxy->invalid_rep, es);
 	} else {
 		es = HA_ATOMIC_XCHG(&proxy->invalid_req, es);
 	}
 	free(es);
-	HA_SPIN_UNLOCK(PROXY_LOCK, &proxy->lock);
+	HA_RWLOCK_WRUNLOCK(PROXY_LOCK, &proxy->lock);
 }
 
 /* Configure all proxies which lack a maxconn setting to use the global one by
@@ -1601,7 +1581,7 @@ void proxy_adjust_all_maxconn()
 	struct switching_rule *swrule1, *swrule2;
 
 	for (curproxy = proxies_list; curproxy; curproxy = curproxy->next) {
-		if (curproxy->state == PR_STSTOPPED)
+		if (curproxy->disabled)
 			continue;
 
 		if (!(curproxy->cap & PR_CAP_FE))
@@ -1645,7 +1625,7 @@ void proxy_adjust_all_maxconn()
 	 * loop above because cross-references are not yet fully resolved.
 	 */
 	for (curproxy = proxies_list; curproxy; curproxy = curproxy->next) {
-		if (curproxy->state == PR_STSTOPPED)
+		if (curproxy->disabled)
 			continue;
 
 		/* If <fullconn> is not set, let's set it to 10% of the sum of
@@ -1675,6 +1655,18 @@ static struct cfg_kw_list cfg_kws = {ILH, {
 	{ CFG_LISTEN, "max-keep-alive-queue", proxy_parse_max_ka_queue },
 	{ CFG_LISTEN, "declare", proxy_parse_declare },
 	{ CFG_LISTEN, "retry-on", proxy_parse_retry_on },
+#ifdef TCP_KEEPCNT
+	{ CFG_LISTEN, "clitcpka-cnt", proxy_parse_tcpka_cnt },
+	{ CFG_LISTEN, "srvtcpka-cnt", proxy_parse_tcpka_cnt },
+#endif
+#ifdef TCP_KEEPIDLE
+	{ CFG_LISTEN, "clitcpka-idle", proxy_parse_tcpka_idle },
+	{ CFG_LISTEN, "srvtcpka-idle", proxy_parse_tcpka_idle },
+#endif
+#ifdef TCP_KEEPINTVL
+	{ CFG_LISTEN, "clitcpka-intvl", proxy_parse_tcpka_intvl },
+	{ CFG_LISTEN, "srvtcpka-intvl", proxy_parse_tcpka_intvl },
+#endif
 	{ 0, NULL, NULL },
 }};
 
@@ -1944,9 +1936,9 @@ static int cli_parse_enable_dyncookie_backend(char **args, char *payload, struct
 	/* Note: this lock is to make sure this doesn't change while another
 	 * thread is in srv_set_dyncookie().
 	 */
-	HA_SPIN_LOCK(PROXY_LOCK, &px->lock);
+	HA_RWLOCK_WRLOCK(PROXY_LOCK, &px->lock);
 	px->ck_opts |= PR_CK_DYNAMIC;
-	HA_SPIN_UNLOCK(PROXY_LOCK, &px->lock);
+	HA_RWLOCK_WRUNLOCK(PROXY_LOCK, &px->lock);
 
 	for (s = px->srv; s != NULL; s = s->next) {
 		HA_SPIN_LOCK(SERVER_LOCK, &s->lock);
@@ -1976,9 +1968,9 @@ static int cli_parse_disable_dyncookie_backend(char **args, char *payload, struc
 	/* Note: this lock is to make sure this doesn't change while another
 	 * thread is in srv_set_dyncookie().
 	 */
-	HA_SPIN_LOCK(PROXY_LOCK, &px->lock);
+	HA_RWLOCK_WRLOCK(PROXY_LOCK, &px->lock);
 	px->ck_opts &= ~PR_CK_DYNAMIC;
-	HA_SPIN_UNLOCK(PROXY_LOCK, &px->lock);
+	HA_RWLOCK_WRUNLOCK(PROXY_LOCK, &px->lock);
 
 	for (s = px->srv; s != NULL; s = s->next) {
 		HA_SPIN_LOCK(SERVER_LOCK, &s->lock);
@@ -2019,10 +2011,10 @@ static int cli_parse_set_dyncookie_key_backend(char **args, char *payload, struc
 	/* Note: this lock is to make sure this doesn't change while another
 	 * thread is in srv_set_dyncookie().
 	 */
-	HA_SPIN_LOCK(PROXY_LOCK, &px->lock);
+	HA_RWLOCK_WRLOCK(PROXY_LOCK, &px->lock);
 	free(px->dyncookie_key);
 	px->dyncookie_key = newkey;
-	HA_SPIN_UNLOCK(PROXY_LOCK, &px->lock);
+	HA_RWLOCK_WRUNLOCK(PROXY_LOCK, &px->lock);
 
 	for (s = px->srv; s != NULL; s = s->next) {
 		HA_SPIN_LOCK(SERVER_LOCK, &s->lock);
@@ -2060,7 +2052,7 @@ static int cli_parse_set_maxconn_frontend(char **args, char *payload, struct app
 	/* OK, the value is fine, so we assign it to the proxy and to all of
 	 * its listeners. The blocked ones will be dequeued.
 	 */
-	HA_SPIN_LOCK(PROXY_LOCK, &px->lock);
+	HA_RWLOCK_WRLOCK(PROXY_LOCK, &px->lock);
 
 	px->maxconn = v;
 	list_for_each_entry(l, &px->conf.listeners, by_fe) {
@@ -2071,7 +2063,7 @@ static int cli_parse_set_maxconn_frontend(char **args, char *payload, struct app
 	if (px->maxconn > px->feconn)
 		dequeue_proxy_listeners(px);
 
-	HA_SPIN_UNLOCK(PROXY_LOCK, &px->lock);
+	HA_RWLOCK_WRUNLOCK(PROXY_LOCK, &px->lock);
 
 	return 1;
 }
@@ -2091,14 +2083,9 @@ static int cli_parse_shutdown_frontend(char **args, char *payload, struct appctx
 	if (!px)
 		return 1;
 
-	if (px->state == PR_STSTOPPED)
+	if (px->disabled)
 		return cli_msg(appctx, LOG_NOTICE, "Frontend was already shut down.\n");
 
-	ha_warning("Proxy %s stopped (cumulated conns: FE: %lld, BE: %lld).\n",
-		   px->id, px->fe_counters.cum_conn, px->be_counters.cum_conn);
-	send_log(px, LOG_WARNING, "Proxy %s stopped (cumulated conns: FE: %lld, BE: %lld).\n",
-	         px->id, px->fe_counters.cum_conn, px->be_counters.cum_conn);
-
 	stop_proxy(px);
 	return 1;
 }
@@ -2119,15 +2106,15 @@ static int cli_parse_disable_frontend(char **args, char *payload, struct appctx
 	if (!px)
 		return 1;
 
-	if (px->state == PR_STSTOPPED)
+	if (px->disabled)
 		return cli_msg(appctx, LOG_NOTICE, "Frontend was previously shut down, cannot disable.\n");
 
-	if (px->state == PR_STPAUSED)
-		return cli_msg(appctx, LOG_NOTICE, "Frontend is already disabled.\n");
+	if (!px->li_ready)
+		return cli_msg(appctx, LOG_NOTICE, "All sockets are already disabled.\n");
 
-	HA_SPIN_LOCK(PROXY_LOCK, &px->lock);
+	HA_RWLOCK_WRLOCK(PROXY_LOCK, &px->lock);
 	ret = pause_proxy(px);
-	HA_SPIN_UNLOCK(PROXY_LOCK, &px->lock);
+	HA_RWLOCK_WRUNLOCK(PROXY_LOCK, &px->lock);
 
 	if (!ret)
 		return cli_err(appctx, "Failed to pause frontend, check logs for precise cause.\n");
@@ -2151,15 +2138,15 @@ static int cli_parse_enable_frontend(char **args, char *payload, struct appctx *
 	if (!px)
 		return 1;
 
-	if (px->state == PR_STSTOPPED)
+	if (px->disabled)
 		return cli_err(appctx, "Frontend was previously shut down, cannot enable.\n");
 
-	if (px->state != PR_STPAUSED)
-		return cli_msg(appctx, LOG_NOTICE, "Frontend is already enabled.\n");
+	if (px->li_ready == px->li_all)
+		return cli_msg(appctx, LOG_NOTICE, "All sockets are already enabled.\n");
 
-	HA_SPIN_LOCK(PROXY_LOCK, &px->lock);
+	HA_RWLOCK_WRLOCK(PROXY_LOCK, &px->lock);
 	ret = resume_proxy(px);
-	HA_SPIN_UNLOCK(PROXY_LOCK, &px->lock);
+	HA_RWLOCK_WRUNLOCK(PROXY_LOCK, &px->lock);
 
 	if (!ret)
 		return cli_err(appctx, "Failed to resume frontend, check logs for precise cause (port conflict?).\n");
@@ -2238,7 +2225,7 @@ static int cli_io_handler_show_errors(struct appctx *appctx)
 	while (appctx->ctx.errors.px) {
 		struct error_snapshot *es;
 
-		HA_SPIN_LOCK(PROXY_LOCK, &appctx->ctx.errors.px->lock);
+		HA_RWLOCK_RDLOCK(PROXY_LOCK, &appctx->ctx.errors.px->lock);
 
 		if ((appctx->ctx.errors.flag & 1) == 0) {
 			es = appctx->ctx.errors.px->invalid_req;
@@ -2348,7 +2335,7 @@ static int cli_io_handler_show_errors(struct appctx *appctx)
 			appctx->ctx.errors.bol = newline;
 		};
 	next:
-		HA_SPIN_UNLOCK(PROXY_LOCK, &appctx->ctx.errors.px->lock);
+		HA_RWLOCK_RDUNLOCK(PROXY_LOCK, &appctx->ctx.errors.px->lock);
 		appctx->ctx.errors.bol = 0;
 		appctx->ctx.errors.ptr = -1;
 		appctx->ctx.errors.flag ^= 1;
@@ -2360,7 +2347,7 @@ static int cli_io_handler_show_errors(struct appctx *appctx)
 	return 1;
 
  cant_send_unlock:
-	HA_SPIN_UNLOCK(PROXY_LOCK, &appctx->ctx.errors.px->lock);
+	HA_RWLOCK_RDUNLOCK(PROXY_LOCK, &appctx->ctx.errors.px->lock);
  cant_send:
 	si_rx_room_blk(si);
 	return 0;
