@@ -50,7 +50,6 @@ __decl_aligned_rwlock(wq_lock);   /* RW lock related to the wait queue */
 #ifdef USE_THREAD
 struct eb_root timers;      /* sorted timers tree, global */
 struct eb_root rqueue;      /* tree constituting the run queue */
-int global_rqueue_size; /* Number of element sin the global runqueue */
 #endif
 
 static unsigned int rqueue_ticks;  /* insertion count */
@@ -96,7 +95,6 @@ void task_kill(struct task *t)
 			thr = my_ffsl(t->thread_mask) - 1;
 
 			/* Beware: tasks that have never run don't have their ->list empty yet! */
-			LIST_INIT(&((struct tasklet *)t)->list);
 			MT_LIST_ADDQ(&task_per_thread[thr].shared_tasklet_list,
 			             (struct mt_list *)&((struct tasklet *)t)->list);
 			_HA_ATOMIC_ADD(&tasks_run_queue, 1);
@@ -151,7 +149,6 @@ void __task_wakeup(struct task *t, struct eb_root *root)
 	eb32sc_insert(root, &t->rq, t->thread_mask);
 #ifdef USE_THREAD
 	if (root == &rqueue) {
-		global_rqueue_size++;
 		_HA_ATOMIC_OR(&t->state, TASK_GLOBAL);
 		HA_SPIN_UNLOCK(TASK_RQ_LOCK, &rq_lock);
 	} else
@@ -193,6 +190,12 @@ void __task_wakeup(struct task *t, struct eb_root *root)
  */
 void __task_queue(struct task *task, struct eb_root *wq)
 {
+#ifdef USE_THREAD
+	BUG_ON((wq == &timers && !(task->state & TASK_SHARED_WQ)) ||
+	       (wq == &sched->timers && (task->state & TASK_SHARED_WQ)) ||
+	       (wq != &timers && wq != &sched->timers));
+#endif
+
 	if (likely(task_in_wq(task)))
 		__task_unlink_wq(task);
 
@@ -284,15 +287,23 @@ void wake_expired_tasks()
 		}
 	}
 	key = eb->key;
-	HA_RWLOCK_RDUNLOCK(TASK_WQ_LOCK, &wq_lock);
 
-	if (tick_is_lt(now_ms, key))
+	if (tick_is_lt(now_ms, key)) {
+		HA_RWLOCK_RDUNLOCK(TASK_WQ_LOCK, &wq_lock);
 		goto leave;
+	}
 
 	/* There's really something of interest here, let's visit the queue */
 
+	if (HA_RWLOCK_TRYRDTOSK(TASK_WQ_LOCK, &wq_lock)) {
+		/* if we failed to grab the lock it means another thread is
+		 * already doing the same here, so let it do the job.
+		 */
+		HA_RWLOCK_RDUNLOCK(TASK_WQ_LOCK, &wq_lock);
+		goto leave;
+	}
+
 	while (1) {
-		HA_RWLOCK_WRLOCK(TASK_WQ_LOCK, &wq_lock);
   lookup_next:
 		if (max_processed-- <= 0)
 			break;
@@ -310,26 +321,29 @@ void wake_expired_tasks()
 		task = eb32_entry(eb, struct task, wq);
 		if (tick_is_expired(task->expire, now_ms)) {
 			/* expired task, wake it up */
+			HA_RWLOCK_SKTOWR(TASK_WQ_LOCK, &wq_lock);
 			__task_unlink_wq(task);
+			HA_RWLOCK_WRTOSK(TASK_WQ_LOCK, &wq_lock);
 			task_wakeup(task, TASK_WOKEN_TIMER);
 		}
 		else if (task->expire != eb->key) {
 			/* task is not expired but its key doesn't match so let's
 			 * update it and skip to next apparently expired task.
 			 */
+			HA_RWLOCK_SKTOWR(TASK_WQ_LOCK, &wq_lock);
 			__task_unlink_wq(task);
 			if (tick_isset(task->expire))
 				__task_queue(task, &timers);
+			HA_RWLOCK_WRTOSK(TASK_WQ_LOCK, &wq_lock);
 			goto lookup_next;
 		}
 		else {
 			/* task not expired and correctly placed */
 			break;
 		}
-		HA_RWLOCK_WRUNLOCK(TASK_WQ_LOCK, &wq_lock);
 	}
 
-	HA_RWLOCK_WRUNLOCK(TASK_WQ_LOCK, &wq_lock);
+	HA_RWLOCK_SKUNLOCK(TASK_WQ_LOCK, &wq_lock);
 #endif
 leave:
 	return;
@@ -444,13 +458,10 @@ unsigned int run_tasks_from_lists(unsigned int budgets[])
 		t->calls++;
 		sched->current = t;
 
-		_HA_ATOMIC_SUB(&tasks_run_queue, 1);
-
 		if (TASK_IS_TASKLET(t)) {
-			LIST_DEL_INIT(&((struct tasklet *)t)->list);
-			__ha_barrier_store();
 			state = _HA_ATOMIC_XCHG(&t->state, state);
 			__ha_barrier_atomic_store();
+			__tasklet_remove_from_tasklet_list((struct tasklet *)t);
 			process(t, ctx, state);
 			done++;
 			sched->current = NULL;
@@ -458,10 +469,9 @@ unsigned int run_tasks_from_lists(unsigned int budgets[])
 			continue;
 		}
 
-		LIST_DEL_INIT(&((struct tasklet *)t)->list);
-		__ha_barrier_store();
 		state = _HA_ATOMIC_XCHG(&t->state, state | TASK_RUNNING);
 		__ha_barrier_atomic_store();
+		__tasklet_remove_from_tasklet_list((struct tasklet *)t);
 
 		/* OK then this is a regular task */
 
@@ -587,7 +597,7 @@ void process_runnable_tasks()
 	 * 100% due to rounding, this is not a problem. Note that while in
 	 * theory the sum cannot be NULL as we cannot get there without tasklets
 	 * to process, in practice it seldom happens when multiple writers
-	 * conflict and rollback on MT_LIST_ADDQ(shared_tasklet_list), causing
+	 * conflict and rollback on MT_LIST_TRY_ADDQ(shared_tasklet_list), causing
 	 * a first MT_LIST_ISEMPTY() to succeed for thread_has_task() and the
 	 * one above to finally fail. This is extremely rare and not a problem.
 	 */
